N              [0-9]
O              ({N}{1,3})

    #include <sys/param.h>
    #include <sys/socket.h>

    #include <netinet/in.h>

    #include <arpa/inet.h>
    #include <arpa/nameser.h>

    #include <ctype.h>
    #include <netdb.h>
    #include <resolv.h>
    #include <stdio.h>
    #include <string.h>
    #include <syslog.h>
    #include <unistd.h>

    #include <lbl/os-proto.h>

    #include "hf.h"

    #ifdef FLEX_SCANNER
    #define YY_NO_UNPUT
    #endif

    /* ANSI C defines this */
    #ifndef __STDC__
    extern char *malloc();
    #endif

    #define ECHO echo(yytext, yyleng)
    #define YY_INPUT(buf, result, max_size) \
	result = read_my_buffer(buf, max_size);
    
    static char crlf[] = "\r\n";

    /* Forwards */
    static char *addr2host __P((u_long));
    static void echo __P((char *, int));
    static int read_my_buffer __P((char *, int));
    int yywrap __P((void));
    static void convert __P((char *, int));
    int yylex __P((void));


%%


{O}\.{O}\.{O}\.{O}	convert(yytext, yyleng);

({O}\.){1,3}		ECHO;		/* anti-backtrack */
{O}((\.{O}){1,2})	ECHO;		/* anti-backtrack */

{N}+			ECHO;
[^0-9\n]+		ECHO;
[^0-9\n]+\n		{
			echo(yytext, yyleng - 1);
			echo(crlf, sizeof(crlf) - 1);
			}

\n			echo(crlf, sizeof(crlf) - 1);

%%

#ifndef lint
static const char rcsid[] =
    "@(#) $Header: hf.l,v 1.8 96/10/15 01:27:28 leres Exp $ (LBL)";
#endif


static	char *outbuf;
static	char *outp;
static	char *inp;
static	u_int outlen;
static	u_int inlen;

#ifndef roundup
#define roundup(x, y) ((((x) + ((y) - 1)) / (y)) * (y))
#endif

char *
hf(str)
	char *str;
{
	static FILE *devnull = NULL;

	if (devnull == NULL) {
		devnull = fopen("/dev/null", "r");
		if (devnull == NULL)
			abort();
	}

	if (outbuf == NULL) {
		outlen = 1024;
		outbuf = malloc(outlen);
		if (outbuf == NULL) {
			syslog(LOG_ERR, "malloc: %m");
			exit(1);
		}
	}

	inp = str;
	inlen = strlen(inp);
	outp = outbuf;
	(void)yylex();
	yyin = yyin == NULL ? devnull : NULL;		/* XXX */
	*outp = '\0';
	return (outbuf);
}

static void
echo(s, n)
	char *s;
	int n;
{

	if (n + 1 >= outlen - (outp - outbuf)) {
		outlen = roundup(outlen + n + 1, 1024);
		outp -= (u_int)outbuf;
		outbuf = realloc(outbuf, outlen);
		if (outbuf == NULL) {
			syslog(LOG_ERR, "malloc: %m");
			exit(1);
		}
		outp += (u_int)outbuf;
	}
	strncpy(outp, s, n);
	outp += n;
}

static int
read_my_buffer(buf, max_size)
	char *buf;
	int max_size;
{
	register int n;

	n = inlen;
	if (n <= 0)
		return (0);

	if (n > max_size)
		n = max_size;

	strncpy(buf, inp, n);
	inp += n;
	inlen -= n;
	return (n);
}

int
yywrap()
{

	return (1);
}

static char *
addr2host(addr)
	u_long addr;
{
	register struct hostent *hp;
	struct in_addr ina;

	/* Try to lookup this host */
	ina.s_addr = addr;
	hp = gethostbyaddr((char *)&ina, sizeof(ina), AF_INET);
	if (hp == NULL)
		return (inet_ntoa(ina));
	return (hp->h_name);
}

static void
convert(str, len)
	char *str;
	int len;
{
	char *host;
	u_long l;

	if ((long)(l = inet_addr(str)) != -1 && (host = addr2host(l)))
		echo(host, strlen(host));
	else
		echo(str, len);
}
