#ifndef lint
static const char rcsid[] =
    "@(#) $Header: noted.c,v 1.12 96/10/15 01:27:14 leres Locked $ (LBL)";
#endif

#include <sys/types.h>
#include <sys/time.h>

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include <lbl/os-proto.h>

#include "wall.h"

char	*prog = "?";
char	*notelog = "/var/log/notelog";
#ifdef HAVE_VAR_RUN
char	*pidfile = "/var/run/noted.pid";
#else
char	*pidfile = "/etc/noted.pid";
#endif
int	notefd = -1;
int	debug;

/* Forwards */
int	main(int, char **);
void	mysleep(u_int);
SIGRET	opennotelog(int);
__dead	void usage(void) __attribute__((volatile));

extern int optind;
extern char *optarg;

int
main(argc, argv)
	int argc;
	char **argv;
{
	register int op, n;
	register char *cp;
	register char **users;
	FILE *fp;
	char buf[1024];

	if ((cp = strrchr(argv[0], '/')) != NULL)
		prog = cp + 1;
	else
		prog = argv[0];
	openlog(prog, 0, LOG_DAEMON);

	while ((op = getopt(argc, argv, "df:")) != EOF)
		switch (op) {

		case 'd':
			++debug;
			break;

		case 'f':
			notelog = optarg;
			break;

		default:
			usage();
		}

	/* Username list is required */
	if (argc == optind)
		usage();
	users = argv + optind;

	/* Signal handler for newsyslog script */
	if ((int)signal(SIGHUP, opennotelog) < 0) {
		syslog(LOG_ERR, "signal: %m");
		exit(1);
	}
	
	/* Submerge if not debugging */
	if (!debug) {
		daemon(0, 0);
		
		/* Update the pid file */
		fp = fopen(pidfile, "w");
		if (fp == NULL)
			syslog(LOG_ERR, "fopen %s: %m", pidfile);
		else {
			(void)fprintf(fp, "%lu\n", (u_long)getpid());
			(void)fclose(fp);
		}
	}

	/* Open the log and seek to the end */
	opennotelog(0);
	if (notefd >= 0)
		(void)lseek(notefd, (off_t)0, SEEK_END);

	for (;;) {
		if (notefd < 0) {
			mysleep(60);
			continue;
		}
		n = read(notefd, buf, sizeof(buf));
		if (n < 0) {
			syslog(LOG_ERR, "read: %m");
			exit(1);
		}
		if (n == 0) {
			mysleep(1);
			continue;
		}
		wall(users, buf, n);
	}
}

SIGRET
opennotelog(signo)
	int signo;
{

	if (notefd >= 0)
		(void)close(notefd);
	notefd = open(notelog, O_RDONLY, 0);
	if (notefd < 0)
		syslog(LOG_ERR, "open %s: %m", notelog);
	return SIGRETVAL;
}

void
mysleep(s)
	register u_int s;
{
	register int n;
	struct timeval tv;

	tv.tv_sec = s;
	tv.tv_usec = 0;
	n = select(0, NULL, NULL, NULL, &tv);
	if (n < 0 && errno != EINTR) {
		syslog(LOG_ERR, "select: %m");
		exit(1);
	}
}

__dead void
usage()
{
	(void)fprintf(stderr, "usage: %s [-d] [-f log] user ...\n", prog);
	exit(1);
}
